package net.sf.jack4j;

/*
Copyright (C) 2008 Ondrej Par

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

import java.nio.Buffer;
import java.nio.ByteBuffer;

/**
 * Port registered by {@link JackClient}.
 * 
 * @author repa
 * 
 */
public class JackLocalPort extends JackPort {

	static {
		JackBridge.initializeJackBridge();
	}

	private JackClient client;
	@SuppressWarnings("unused")
	private long clientHandle = 0;
	@SuppressWarnings("unused")
	private ByteBuffer cachedByteBuffer = null;

	JackLocalPort(long portHandle, JackClient client) {
		super(portHandle);
		this.client = client;
		this.clientHandle = client.getClientHandle();
	}

	/**
	 * Returns the client that owns this port.
	 */
	public JackClient getClient() {
		return client;
	}

	/**
	 * Sets short name of this port.
	 */
	public void setPortName(String portName) throws JackException {
		synchronized (client) {
			setPortNameInternal(portName);
			client.localPortNameChanged(this, portName);
		}
	}

	private native void setPortNameInternal(String portName) throws JackException;

	/**
	 * Sets port alias.
	 * 
	 * <p>
	 * The port can have at most 2 aliases.
	 */
	public native void setPortAlias(String alias) throws JackException;

	/**
	 * Removes port alias.
	 */
	public native void unsetPortAlias(String alias) throws JackException;

	/**
	 * Returns all aliases set for this port.
	 */
	public native String[] getPortAliases() throws JackException;

	/**
	 * Returns number of connections to or from port.
	 */
	public native int connected() throws JackException;

	/**
	 * Returns true if the port is directly connected to port with specified
	 * name.
	 */
	public native boolean connectedTo(String portName) throws JackException;

	/**
	 * Returns full port names of to which this port is connected.
	 * 
	 * <p>
	 * If the port is not connected, returns null.
	 */
	public native String[] getConnections() throws JackException;

	/**
	 * Returns pointer to the memory area associated with the specified port.
	 * 
	 * <p>
	 * The pointer is not very useful in Java, but can be used for some JNI
	 * calls.
	 * 
	 * <p>
	 * <b>This method can only be called during {@link JackClient#process(int)}
	 * callback, and only by the thread that invoked the callback. The returned
	 * value is valid only during that callback.</b>
	 * 
	 * @param bufferSize
	 *            the value passed in <code>bufferSize</code> parameter to
	 *            {@link JackClient#process(int)}
	 * @return native pointer, converted to long @
	 */
	public native long getBufferPointer(int bufferSize) throws JackException;

	/**
	 * Returns ByteBuffer that contains the memory area associated with the
	 * specified port.
	 * 
	 * <p>
	 * <b>This method can only be called during {@link JackClient#process(int)}
	 * callback, and only by the thread that invoked the callback. The returned
	 * value is valid only during that callback.</b>
	 * 
	 * <p>
	 * The caller is responsible for calling {@link Buffer#clear()} on the
	 * returned value before the buffer is used. Otherwise, <b>the buffer
	 * position, limit, and mark may be set to random value</b>. The buffer
	 * capacity is always set to the buffer size multiplied by
	 * <code>Float.SIZE / 8</code>.
	 * 
	 * @param bufferSize
	 *            the value passed in <code>bufferSize</code> parameter to
	 *            {@link JackClient#process(int)}
	 * @return port buffer, wrapped into ByteBuffer object
	 */
	public native ByteBuffer getByteBuffer(int bufferSize) throws JackException;

	/**
	 * Ties this (output) port to given input port.
	 * 
	 * <p>
	 * The sourcePort must be owned by the same client as this port, and it must
	 * be an input port.
	 * 
	 * <p>
	 * This port must be an output port.
	 */
	public native void tie(JackLocalPort sourcePort) throws JackException;

	/**
	 * Undoes the effect of {@link #tie(JackLocalPort)}.
	 */
	public native void untie() throws JackException;
}
