package net.sf.jack4j;

/*
Copyright (C) 2008 Ondrej Par

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

/**
 * MIDI buffer for {@link JackLocalMidiPort}.
 * 
 * <p>
 * The instance may be obtained with
 * {@link JackLocalMidiPort#initializeMidiBuffer(int)}, and is only valid
 * <b>during the current process cycle</b>.
 */
public class JackMidiPortBuffer {

	private long bufferPointer;

	public JackMidiPortBuffer(long bufferPointer) {
		this.bufferPointer = bufferPointer;
	}

	/**
	 * Native pointer to buffer data.
	 * 
	 * <p>
	 * The pointer is packed as <code>long</code> value.
	 */
	public long getBufferPointer() {
		return bufferPointer;
	}

	/**
	 * To be used solely by {@link JackLocalMidiPort}.
	 */
	void setBufferPointer(long bufferPointer) {
		this.bufferPointer = bufferPointer;
	}

	/**
	 * Returns current number of MIDI events in the buffer.
	 */
	public native int getEventCount();

	/**
	 * Returns MIDI event from the (input) buffer.
	 * 
	 * @param index
	 *            event number, must be non-negative and lower than
	 *            {@link #getEventCount()}
	 * @return the event
	 * @throws JackException
	 *             if native Jack API fails
	 */
	public native JackMidiEvent getEvent(int index) throws JackException;

	/**
	 * Clears an event buffer.
	 * 
	 * <p>
	 * May be called only during client's process callback.
	 * 
	 * <p>
	 * This should be called at the beginning of each process cycle before
	 * calling {@link #eventReserve(int, int)} or
	 * {@link #eventWrite(int, byte[])}. This function may not be called on an
	 * input port.
	 */
	public native void clearBuffer();

	/**
	 * Allocate space for an event to be written to an event port buffer.
	 * 
	 * <p>
	 * May be called only during client's process callback.
	 * 
	 * <p>
	 * There's probably no good reason to use this method in Java; use
	 * {@link #eventWrite(int, byte[])} instead.
	 * 
	 * <p>
	 * The returned value is a native pointer of type
	 * <code>jack_midi_data_t*</code>. packed as long.
	 * 
	 * <p>
	 * Clients are to write the actual event data to be written starting at the
	 * pointer returned by this function. Clients must not write more than
	 * dataSize bytes into this buffer. Clients must write normalized MIDI data
	 * to the port - no running status and no (1-byte) realtime messages
	 * interspersed with other messages (realtime messages are fine when they
	 * occur on their own, like other messages).
	 * 
	 * @param time
	 *            sample offset of the event (number of frames since cycle
	 *            start)
	 * @param dataSize
	 *            length of event's raw data in bytes
	 * @return <code>jack_midi_data_t*</code> packed as long
	 * @throws JackException
	 *             if the error occurrs (native function
	 *             <code>jack_midi_event_reserve</code> returns NULL)
	 */
	public native long eventReserve(int time, int dataSize) throws JackException;

	/**
	 * Write an event into an event port buffer.
	 * 
	 * <p>
	 * May be called only during client's process callback.
	 * 
	 * <p>
	 * The <code>data</code> must be normalized MIDI data - no running status
	 * and no (1-byte) realtime messages interspersed with other messages
	 * (realtime messages are fine when they occur on their own, like other
	 * messages).
	 * 
	 * @param time
	 *            sample offset of the event (number of frames since cycle
	 *            start)
	 * @param data
	 *            raw MIDI data
	 * @throws JackException
	 *             if error occurrs
	 */
	public native void eventWrite(int time, byte[] data) throws JackException;

	/**
	 * Not for public use, intended for unit tests.
	 * 
	 * <p>
	 * Calls {@link #eventReserve(int, int)} and then writes given data to the
	 * reserved space.
	 */
	native void eventReserveAndWrite(int time, byte[] data) throws JackException;

	/**
	 * Returns the size of the largest event that can be stored by the port.
	 * 
	 * <p>
	 * May be called only during client's process callback.
	 */
	public native int getMaxEventSize();

	/**
	 * Returns the number of events that could not be written to port.
	 */
	public native int getLostEventsCount();

}
