package net.sf.jack4j;

/*
Copyright (C) 2008 Ondrej Par

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

/**
 * JackClient with additional transport-related methods.
 * 
 * @author repa
 * 
 */
public abstract class JackTransportClient extends JackClient {

	public JackTransportClient(String clientName, boolean useExactName, boolean canStartServer, String serverName)
	        throws JackException {
		super(clientName, useExactName, canStartServer, serverName);

		populateCallbackStructure();
	}

	/**
	 * Starts the Jack transport.
	 */
	public native void startTransport();

	/**
	 * Stops the Jack transport.
	 */
	public native void stopTransport();

	/**
	 * Queries current state and position of Jack transport.
	 * 
	 * <p>
	 * If the <code>position</code> parameter is not null, the given structure
	 * will be updated with the current positional information.
	 */
	public native JackTransportState queryTransport(TransportPosition position);

	/**
	 * Return an estimate of the current transport frame, including any time
	 * elapsed since the last transport positional update.
	 */
	public native int getCurrentTransportFrame();

	/**
	 * Reposition the transport to a new frame number.
	 * 
	 * <p>
	 * May be May be called at any time by any client. The new position takes
	 * effect in two process cycles. If there are slow-sync clients and the
	 * transport is already rolling, it will enter the
	 * {@link JackTransportState#STARTING} state and begin invoking their
	 * synchronization callbacks until ready. This function is realtime-safe.
	 */
	public native void locateTransport(int frame) throws JackException;

	/**
	 * Request a new transport position.
	 * 
	 * <p>
	 * May be May be called at any time by any client. The new position takes
	 * effect in two process cycles. If there are slow-sync clients and the
	 * transport is already rolling, it will enter the
	 * {@link JackTransportState#STARTING} state and begin invoking their
	 * synchronization callbacks until ready. This function is realtime-safe.
	 */
	public native void repositionTransport(TransportPosition position) throws JackException;

	/**
	 * Set the timeout value for slow-sync clients.
	 */
	public native void setSyncTimeout(long microseconds) throws JackException;

	/**
	 * Declares this client as a slow-sync client, using default callback.
	 * 
	 * <p>
	 * Setting up a sync callback declares this client as a slow-sync client,
	 * one that cannot respond immediately to transport position changes.
	 * 
	 * <p>
	 * After the default callback is set, the method
	 * {@link #syncCallback(JackTransportState, TransportPosition)} will be
	 * invoked accordingly to Jack synchronization rules (see the description of
	 * <code>syncCallback</code> method).
	 */
	public native void setDefaultSyncCallback() throws JackException;

	/**
	 * Declares this client as a slow-sync client, using default callback.
	 * 
	 * <p>
	 * Setting up a sync callback declares this client as a slow-sync client,
	 * one that cannot respond immediately to transport position changes.
	 * 
	 * <p>
	 * The <code>pointer</code> must be a native <code>JackSyncCallback</code>
	 * pointer, represented as <code>long</code>.
	 * 
	 * @param pointer
	 *            the pointer to native function
	 * @param arg
	 *            the value passed as <code>void* arg</code> parameter to the
	 *            callback
	 * @see #setDefaultSyncCallback()
	 */
	public native void setSyncCallback(long pointer, long arg) throws JackException;

	/**
	 * Declares that the client is not slow-sync anymore, and unregisters native
	 * sync callback.
	 * 
	 * <p>
	 * After this method is called, the
	 * {@link #syncCallback(JackTransportState, TransportPosition)} method won't
	 * be called anymore.
	 */
	public native void unsetSyncCallback() throws JackException;

	/**
	 * Registers this client as timebase master for the Jack.
	 * 
	 * <p>
	 * The timebase master registers a callback that updates extended position
	 * information such as beats or timecode whenever necessary. Without this
	 * extended information, there is no need for this function.
	 * 
	 * <p>
	 * There is never more than one master at a time. When a new client takes
	 * over, the former timebase callback is no longer called. Taking over the
	 * timebase may be done conditionally, so it fails if there was a master
	 * already.
	 * 
	 * @param conditional
	 *            if true, the request is made conditionally, ie. if there
	 *            already is a timebase master, the method does nothing
	 * @param pointer
	 *            the pointer to native function
	 * @param arg
	 *            the value passed as <code>void* arg</code> parameter to the
	 *            callback
	 * @return true if the call was successful; false if conditional request was
	 *         made and there's already timebase master.
	 * @see #setDefaultTimebaseCallback(boolean)
	 */
	public native boolean setTimebaseCallback(boolean conditional, long pointer, long arg) throws JackException;

	/**
	 * Registers this client as timebase master for the Jack, using default
	 * callback.
	 * 
	 * <p>
	 * The timebase master registers a callback that updates extended position
	 * information such as beats or timecode whenever necessary. Without this
	 * extended information, there is no need for this function.
	 * 
	 * <p>
	 * There is never more than one master at a time. When a new client takes
	 * over, the former timebase callback is no longer called. Taking over the
	 * timebase may be done conditionally, so it fails if there was a master
	 * already.
	 * 
	 * <p>
	 * The default timebase callback invokes the method
	 * {@link #timebaseCallback(JackTransportState, int, TransportPosition, boolean)}
	 * of this client.
	 * 
	 * @param conditional
	 *            if true, the request is made conditionally, ie. if there
	 *            already is a timebase master, the method does nothing
	 * @return true if the call was successful; false if conditional request was
	 *         made and there's already timebase master.
	 */
	public native boolean setDefaultTimebaseCallback(boolean conditional) throws JackException;

	/**
	 * Releases this client from timebase master responsibility.
	 * 
	 * <p>
	 * After this call, this Jack client is not timebase master anymore. The
	 * transport state keeps rolling if it was started, but no extended
	 * positional information will be available. The
	 * {@link #timebaseCallback(JackTransportState, int, TransportPosition, boolean)}
	 * will not be invoked anymore.
	 * 
	 * @see #setDefaultTimebaseCallback(boolean)
	 */
	public native void releaseTimebase() throws JackException;

	/**
	 * Synchronization callback for slow-sync clients.
	 * 
	 * <p>
	 * <b>The method is never invoked until the
	 * {@link #setDefaultSyncCallback()} is called.</b>
	 * 
	 * <p>
	 * After the default slow-sync callback is set up, this method will be
	 * called during the next process cycle (if the client is active), or after
	 * the call to {@link JackClient#activate()} (if the client was previously
	 * inactive).
	 * 
	 * <p>
	 * Then, the method is called again whenever the transport starts, and
	 * whenever some client requests new transport position.
	 * 
	 * <p>
	 * The method must return boolean value that indicates whether the client is
	 * ready to roll.
	 * 
	 * <p>
	 * <b>The <code>position</code> structure is only valid during the call.</b>
	 * If you need the positional information outside the callback, you can
	 * create a clone with {@link TransportPosition#clone()}.
	 * 
	 * @param state
	 *            current transport state
	 * @param position
	 *            new transport position
	 * @return true if the client is ready, false otherwise.
	 */
	public abstract boolean syncCallback(JackTransportState state, TransportPosition position);

	/**
	 * Callback that provides extended position information to Jack transport.
	 * 
	 * <p>
	 * This method is never invoked unless the
	 * {@link #setDefaultTimebaseCallback(boolean)} was successfully called.
	 * 
	 * <p>
	 * This method is called immediately after <code>proces</code> callback in
	 * the same thread whenever the transport is rolling, or when any client has
	 * requested a new position in the previous cycle. The first cycle after the
	 * (default) native timebase callback was registered is also treated as a
	 * new position, or the first cycle after {@link #activate()} if the client
	 * had been inactive.
	 * 
	 * <p>
	 * The method is responsible for setting extended position information into
	 * <code>pos</code> structure. The actual transport position can not be
	 * set using <code>frame</code> field of that structure; use
	 * {@link #locateTransport(int)} instead. <b>The <code>position</code>
	 * structure is only valid during the call.</b> If you need the positional
	 * information outside the callback, you can create a clone with
	 * {@link TransportPosition#clone()}.
	 * 
	 * @param state
	 *            current transport state
	 * @param nframes
	 *            period length
	 * @param pos
	 *            position structure for the <i>next</i> cycle; if newPos is
	 *            false, the structure already contains extended positional
	 *            information from current cycle
	 * @param newPos
	 *            if true, new position was requested; in such case, pos
	 *            structure doesn't contain valid extended information
	 */
	public abstract void timebaseCallback(JackTransportState state, int nframes, TransportPosition pos, boolean newPos);

	private native void populateCallbackStructure();
}
